"use strict";
let Bluebird = require('bluebird');
let debug = require('debug')('cached-method-proxy');
const Immutable = require('immutable');
let isPromise = require('is-promise');
let LRU = require('lru-cache');
class CachedFunctionProxy {
    constructor(proxiedFunction, thisArg) {
        this.proxiedFunction = proxiedFunction;
        this.thisArg = thisArg;
        this.cacheKeySeq = 0;
        this.cacheKeys = Immutable.Map();
        this.call = this.proxyCall.bind(this);
        this.cache = LRU();
    }
    proxyCall() {
        let args = Array.from(arguments);
        let keyObject = Immutable.fromJS(args);
        let cacheKey = this.cacheKeys.get(keyObject);
        if (cacheKey !== undefined) {
            let cacheRecord = this.getCacheRecord(cacheKey);
            if (cacheRecord !== undefined && cacheRecord.bluebirdPromise !== undefined) {
                // Don't cache rejected promises
                if (!cacheRecord.bluebirdPromise.isRejected()) {
                    return cacheRecord.value;
                }
            }
            else if (cacheRecord !== undefined) {
                return cacheRecord.value;
            }
        }
        cacheKey = this.cacheKeySeq++;
        this.cacheKeys = this.cacheKeys.set(keyObject, cacheKey);
        let value = this.proxiedFunction.apply(this.thisArg, args);
        this.putCacheRecord(cacheKey, value);
        return value;
    }
    ;
    clear() {
        this.cacheKeys = Immutable.Map();
        this.cache.reset();
    }
    getCacheRecord(key) {
        return this.cache.get(key);
    }
    putCacheRecord(key, value) {
        if (isPromise(value)) {
            this.cache.set(key, {
                value: value,
                bluebirdPromise: Bluebird.resolve(value)
            });
        }
        else {
            this.cache.set(key, { value: value });
        }
    }
}
exports.CachedFunctionProxy = CachedFunctionProxy;
class CachedMethodProxy {
    constructor(proxiedObject, options = {}) {
        this.proxiedObject = proxiedObject;
        this.options = options;
        this.excludedMethods = new Set(options.exclude);
        let self = this;
        this.methodProxies = new Map();
        this.proxy = new Proxy(proxiedObject, {
            get(target, propKey, receiver) {
                const origProp = target[propKey];
                if (typeof origProp !== 'function') {
                    return origProp;
                }
                else if (!self.checkInclusion(origProp)) {
                    return origProp;
                }
                if (!self.methodProxies.has(propKey)) {
                    debug(`Adding method proxy for method: ${propKey}`);
                    self.methodProxies.set(propKey, new CachedFunctionProxy(origProp, proxiedObject));
                }
                debug(`Returning proxied method: ${propKey}`);
                return self.methodProxies.get(propKey).call;
            }
        });
    }
    clear() {
        this.methodProxies.forEach(proxy => proxy.clear());
    }
    checkInclusion(method) {
        return !this.excludedMethods.has(method);
    }
}
exports.CachedMethodProxy = CachedMethodProxy;
//# sourceMappingURL=cached_method_proxy.js.map